
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>




using namespace std;

#define OUTPUTFILENAME		"%%tmp_ipmitool_output"

const int ipmbaddr[] = {0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0xa4};
const char* arglist[] = {"?", "-s", "-num", "-msk"};
enum argtype {HELP=0, SLOT, SNUM, MSKVAL, LASTARG};
char cmdlinebuf[256];


int main(int argc, char** argv) {
  bool fatal = false;
  int slotnum = 0;
  unsigned char retbytes[32];
  int retbytecnt;
  FILE* rfile;
  long faultmasklevel = -1;
  long sensornum = -1;

  // parse command line
  int i1 = 1;
  while ((i1<argc) && !fatal) {
    int argidx;
    string curargv = argv[i1];
    for (argidx=0; argidx<LASTARG; argidx++) {
      if (curargv == arglist[argidx]) {
        // have match
        uint tempui;
        switch (argidx) {
          case HELP:
            cout << "command line arguments:\n";
            cout << "  ? - display this text\n";
            cout << "  -s <slot#> - MicroTCA slot number (1-13)\n";
            cout << "  -num <sensor#> - Module-specific Sensor Number\n";
            cout << "  -msk <0-3> - Optional mask value to set (otherwise return current setting)\n";
            exit(0);
          case SLOT:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &slotnum);
            }
            break;
          case SNUM:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing sensor number argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &sensornum);
              if ((sensornum < 0) || (sensornum > 255)) {
                cerr << "?error illegal value '" << argv[i1] << "' on switch " << curargv << "\n";
                fatal = true;
              }
            }
            break;

          case MSKVAL:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing mask value argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &faultmasklevel);
              if ((faultmasklevel < 0) || (faultmasklevel > 3)) {
                cerr << "?error unknown value '" << argv[i1] << "' on switch " << curargv << "\n";
                fatal = true;
              }
            }
            break;

          default:
            cerr << "?fatal error while parsing command line--try running command again with '?' argument\n";
            fatal = true;
            break;
        }
        break;
      }
    }
    if (argidx>=LASTARG) {
      cerr << "?error, unknown argument '" << curargv << "'\n";    
      fatal = true;
      break;
    }
    i1++;
  }
  
  if ((slotnum < 1) || (slotnum >13)) {
    cerr << "?error invalid slot number (" << slotnum << ")\n";
    fatal = true;
  }

  if (sensornum == -1) {
    cerr << "?error invalid or undefined sensor number\n";
    fatal = true;
  }


  if (fatal)
    // stop now, before things go too far
    exit(0);

  // check if this is a set or get by whether the mask value is set or not
  if (faultmasklevel == -1) {
    // read current value
    sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x0C 0x%02x >%s\n", ipmbaddr[slotnum], sensornum, OUTPUTFILENAME);
    system(cmdlinebuf);
    // open file and capture the returned bytes
    rfile = fopen(OUTPUTFILENAME, "r");
    if (!rfile) {
      cerr << "?error no results returned on IPMI command\n";
      exit(0);
    }
    retbytecnt = 0;
    while (fscanf(rfile, "%x", &retbytes[retbytecnt]) != EOF) {
      retbytecnt++;
    } 
    fclose(rfile);
    if (retbytecnt != 1) {
      cerr << "?error unexpected number of data bytes returned\n";
      exit(0);
    }
    cout << "Sensor " << sensornum << " Alarm Mask Level: ";
    switch (retbytes[0]) {
      case 0:
        cout << "none\n";
        break;
      case 1:
        cout << "non-critical\n";
        break;
      case 2:
        cout << "critical\n";
        break;
      case 3:
        cout << "nonrecoverable\n";
        break;
      default:
        cout << "unknown\n";
        break;
    }
  }
  else {
    // set new value
    sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x0B 0x%02x 0x%02x >%s\n", ipmbaddr[slotnum], sensornum, faultmasklevel, OUTPUTFILENAME);
    system(cmdlinebuf);
  }
  
  return 1;
}

