
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>




using namespace std;

#define OUTPUTFILENAME		"%%tmp_ipmitool_output"

const int ipmbaddr[] = {0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0xa4};
const char* arglist[] = {"?", "-s", "mmc", "cold", "warm"};
enum argtype {HELP=0, SLOT, MMCRST, COLDRST, WARMRST, LASTARG};
typedef enum {MMC, COLD, WARM} rsttarget_t;
char cmdlinebuf[128];


int main(int argc, char** argv) {
  bool fatal = false;
  int slotnum = 0;
  unsigned char retbytes[32];
  int retbytecnt;
  FILE* rfile;
  rsttarget_t rsttarget = MMC;
  

  // parse command line
  int i1 = 1;
  while ((i1<argc) && !fatal) {
    int argidx;
    string curargv = argv[i1];
    for (argidx=0; argidx<LASTARG; argidx++) {
      if (curargv == arglist[argidx]) {
        // have match
        uint tempui;
        switch (argidx) {
          case HELP:
            cout << "command line arguments:\n";
            cout << "  ? - display this text\n";
            cout << "   -s <slot#> - MicroTCA slot number (1-13)\n";
            cout << "  mmc|cold|warm\n";
            exit(0);
          case SLOT:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else {
              sscanf(argv[i1], "%i", &slotnum);
            }            
            break;

          case MMCRST:
            rsttarget = MMC;
            break;

          case COLDRST:
            rsttarget = COLD;
            break;

          case WARMRST:
            rsttarget = WARM;
            break;

          default:
            cerr << "?fatal error while parsing command line--try running command again with '?' argument\n";
            fatal = true;
            break;
        }
        break;
      }
    }
    if (argidx>=LASTARG) {
      cerr << "?error, unknown argument '" << curargv << "'\n";    
      fatal = true;
      break;
    }
    i1++;
  }
  
  if ((slotnum < 1) || (slotnum >13)) {
    cerr << "?error invalid slot number (" << slotnum << ")\n";
    fatal = true;
  }

  if (fatal)
    // stop now, before things go too far
    exit(0);

  // issue specified reset
  if (rsttarget==MMC) {
    cout << "Resetting MMC in specified slot\n";
    sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x06 0x02 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
  }
  else {
    if (rsttarget==COLD) {
      cout << "Performing Cold Reset of AMC back end\n";
      sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x2c 0x04 0x00 0x00 0x00>%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
    }
    else {
      cout << "Performing Warm Reset of AMC back end\n";
      sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x2c 0x04 0x00 0x00 0x01>%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
    }
  }
  system(cmdlinebuf);

  return 1;

}

