
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>




using namespace std;

#define OUTPUTFILENAME		"%%tmp_ipmitool_output"

const int ipmbaddr[] = {0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0xa4};
const char* arglist[] = {"?", "-s", "t", "m"};
enum argtype {HELP=0, SLOT, TARG, MARG, LASTARG};
char cmdlinebuf[128];


int main(int argc, char** argv) {
  bool fatal = false;
  bool setmodeflag = false;
  bool triggermode;
  int slotnum = 0;
  unsigned char retbytes[32];
  int retbytecnt;
  FILE* rfile;
  

  // parse command line
  int i1 = 1;
  while ((i1<argc) && !fatal) {
    int argidx;
    string curargv = argv[i1];
    for (argidx=0; argidx<LASTARG; argidx++) {
      if (curargv == arglist[argidx]) {
        // have match
        uint tempui;
        switch (argidx) {
          case HELP:
            cout << "command line arguments:\n";
            cout << "  ? - display this text\n";
            cout << " -s <slot#> - MicroTCA slot number (1-13)\n";
            cout << " [t|m]\n";
            exit(0);
          case SLOT:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &slotnum);
            }
            break;
          case TARG:
            setmodeflag = true;
            triggermode = true;
            break;

          case MARG:
            setmodeflag = true;
            triggermode = false;
            break;

          default:
            cerr << "?fatal error while parsing command line--try running command again with '?' argument\n";
            fatal = true;
            break;
        }
        break;
      }
    }
    if (argidx>=LASTARG) {
      cerr << "?error, unknown argument '" << curargv << "'\n";    
      fatal = true;
      break;
    }
    i1++;
  }
  
  if ((slotnum < 1) || (slotnum >13)) {
    cerr << "?error invalid slot number (" << slotnum << ")\n";
    fatal = true;
  }

  if (fatal)
    // stop now, before things go too far
    exit(0);

  // check flags to see if this is query of boot mode or cmd to change settings
  if (setmodeflag) {
    // setting power state
    if (triggermode)
      sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x06 0x00 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
    else
      sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x06 0x01 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
    system(cmdlinebuf);
  }
  else {
    // querying the boot mode state
    // execute IPMI tool on bash command line
    sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x07 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
    system(cmdlinebuf);
    // open file and capture the returned bytes
    rfile = fopen(OUTPUTFILENAME, "r");
    if (!rfile) {
      cerr << "?error no results returned on IPMI command\n";
      exit(0);
    }
    retbytecnt = 0;
    while (fscanf(rfile, "%x", &retbytes[retbytecnt]) != EOF) {
      retbytecnt++;
    } 
    fclose(rfile);
    if (retbytecnt != 1) {
      cerr << "?error unexpected number of data bytes returned\n";
      exit(0);
    }
    cout << "Current Boot Mode: ";
    if (retbytes[0])
      cout << "maintenance\n";
    else
      cout << "trigger\n";
  } 

  return 1;
}
