
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>




using namespace std;

#define OUTPUTFILENAME		"%%tmp_ipmitool_output"

const int ipmbaddr[] = {0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0xa4};
const char* arglist[] = {"?", "-s"};
enum argtype {HELP=0, SLOT, LASTARG};
char cmdlinebuf[128];


int main(int argc, char** argv) {
  bool fatal = false;
  int slotnum = 0;
  unsigned long uptime;
  unsigned long hottime;
  unsigned long days, hours, mins, secs;
  time_t mmc_systime;
  unsigned char retbytes[32];
  int retbytecnt;
  FILE* rfile;
  

  // parse command line
  int i1 = 1;
  while ((i1<argc) && !fatal) {
    int argidx;
    string curargv = argv[i1];
    for (argidx=0; argidx<LASTARG; argidx++) {
      if (curargv == arglist[argidx]) {
        // have match
        uint tempui;
        switch (argidx) {
          case HELP:
            cout << "command line arguments:\n";
            cout << "  ? - display this text\n";
            cout << " -s <slot#> - MicroTCA slot number (1-13)\n";
            fatal = true;
            break;
          case SLOT:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &slotnum);
            }
            break;
          default:
            cerr << "?fatal error while parsing command line--try running command again with '?' argument\n";
            fatal = true;
            break;
        }
        break;
      }
    }
    if (argidx>=LASTARG) {
      cerr << "?error, unknown argument '" << curargv << "'\n";    
      fatal = true;
      break;
    }
    i1++;
  }
  
  if ((slotnum < 1) || (slotnum >13)) {
    cerr << "?error invalid slot number (" << slotnum << ")\n";
    fatal = true;
  }

  if (fatal)
    // stop now, before things go too far
    exit(0);

  // get payload manager setting values
  // execute IPMI tool on bash command line
  sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x04 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
  system(cmdlinebuf);
  // open file and capture the returned bytes
  rfile = fopen(OUTPUTFILENAME, "r");
  if (!rfile) {
    cerr << "?error no results returned on IPMI command\n";
    exit(0);
  }
  retbytecnt = 0;
  while (fscanf(rfile, "%x", &retbytes[retbytecnt]) != EOF) {
    retbytecnt++;
  } 
  fclose(rfile);
  if (retbytecnt != 5) {
    cerr << "?error unexpected number of data bytes returned\n";
    exit(0);
  }
  cout << "Auto Power-On: ";
  if (retbytes[0])
    cout << "enabled\n";
  else
    cout << "disabled\n";
  cout << "Boot Mode: ";
  if (retbytes[1])
    cout << "maintenance\n";
  else
    cout << "trigger\n";
  cout << "FPGA Auto-Config Inhibit: ";
  if (retbytes[2])
    cout << "enabled\n";
  else
    cout << "disabled\n";
  cout << "IPMI Event Generation on Back End Power Shutdown: ";
  if (retbytes[3])
    cout << "enabled\n";
  else
    cout << "disabled\n";
  cout << "Back End Monitor Mask Level: ";
  switch (retbytes[4]) {
    case 0:
      cout << "none\n";
      break;
    case 1:
      cout << "non-critical\n";
      break;
    case 2:
      cout << "critical\n";
      break;
    case 3:
      cout << "nonrecoverable\n";
      break;
    default:
      cout << "unknown\n";
      break;
  }

  return 1;
}
