
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>




using namespace std;

#define OUTPUTFILENAME		"%%tmp_ipmitool_output"

const int ipmbaddr[] = {0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0xa4};
const char* arglist[] = {"?", "-s"};
enum argtype {HELP=0, SLOT, LASTARG};
char cmdlinebuf[128];


int main(int argc, char** argv) {
  bool fatal = false;
  int slotnum = 0;
  unsigned long uptime;
  unsigned long hottime;
  unsigned long days, hours, mins, secs;
  time_t mmc_systime;
  unsigned char retbytes[32];
  int retbytecnt;
  FILE* rfile;
  

  // parse command line
  int i1 = 1;
  while ((i1<argc) && !fatal) {
    int argidx;
    string curargv = argv[i1];
    for (argidx=0; argidx<LASTARG; argidx++) {
      if (curargv == arglist[argidx]) {
        // have match
        uint tempui;
        switch (argidx) {
          case HELP:
            cout << "command line arguments:\n";
            cout << "  ? - display this text\n";
            cout << " -s <slot#> - MicroTCA slot number (1-13)\n";
            exit(0);
          case SLOT:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &slotnum);
            }
            break;
          default:
            cerr << "?fatal error while parsing command line--try running command again with '?' argument\n";
            fatal = true;
            break;
        }
        break;
      }
    }
    if (argidx>=LASTARG) {
      cerr << "?error, unknown argument '" << curargv << "'\n";    
      fatal = true;
      break;
    }
    i1++;
  }
  
  if ((slotnum < 1) || (slotnum >13)) {
    cerr << "?error invalid slot number (" << slotnum << ")\n";
    fatal = true;
  }

  if (fatal)
    // stop now, before things go too far
    exit(0);

  // get up time and hot time values
  // execute IPMI tool on bash command line
  sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x28 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
  system(cmdlinebuf);
  // open file and capture the returned bytes
  rfile = fopen(OUTPUTFILENAME, "r");
  if (!rfile) {
    cerr << "?error no results returned on IPMI command\n";
    exit(0);
  }
  retbytecnt = 0;
  while (fscanf(rfile, "%x", &retbytes[retbytecnt]) != EOF) {
    retbytecnt++;
  } 
  fclose(rfile);
  if (retbytecnt != 8) {
    cerr << "?error unexpected number of data bytes returned\n";
    exit(0);
  }
  uptime = retbytes[3];
  uptime = (uptime << 8) + retbytes[2];
  uptime = (uptime << 8) + retbytes[1];
  uptime = (uptime << 8) + retbytes[0];

  days = uptime/(24*3600);
  uptime -= days*24*3600;
  hours = uptime/3600;
  uptime -= hours*3600;
  mins = uptime/60;
  secs = uptime-mins*60;
  cout << "MMC Up Time: " << days << " Days " << hours << " Hours " << mins << " Minutes " << secs << " Seconds\n";


  hottime = retbytes[7];
  hottime = (hottime << 8) + retbytes[6];
  hottime = (hottime << 8) + retbytes[5];
  hottime = (hottime << 8) + retbytes[4];
  days = hottime/(24*3600);
  hottime -= days*24*3600;
  hours = hottime/3600;
  hottime -= hours*3600;
  mins = hottime/60;
  secs = hottime-mins*60;
  cout << "MMC Hot Time: " << days << " Days " << hours << " Hours " << mins << " Minutes " << secs << " Seconds\n";

  // get system time
  // execute IPMI tool on bash command line
  sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x2a >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
  system(cmdlinebuf);
  // open file and capture the returned bytes
  rfile = fopen(OUTPUTFILENAME, "r");
  if (!rfile) {
    cerr << "?error no results returned on IPMI command\n";
    exit(0);
  }
  retbytecnt = 0;
  while (fscanf(rfile, "%x", &retbytes[retbytecnt]) != EOF) {
    retbytecnt++;
  } 
  fclose(rfile);
  if (retbytecnt != 4) {
    cerr << "?error unexpected number of data bytes returned\n";
    exit(0);
  }
  mmc_systime = retbytes[3];
  mmc_systime = (mmc_systime << 8) + retbytes[2];
  mmc_systime = (mmc_systime << 8) + retbytes[1];
  mmc_systime = (mmc_systime << 8) + retbytes[0];
  printf("MMC System Time:  %s\n",ctime(&mmc_systime));

  return 1;
}
