
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>




using namespace std;

#define OUTPUTFILENAME		"%%tmp_ipmitool_output"

const int ipmbaddr[] = {0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0xa4};
const char* arglist[] = {"?", "-s"};
enum argtype {HELP=0, SLOT, LASTARG};
char cmdlinebuf[128];

typedef struct {
  unsigned char	fmtversion;
  unsigned long eepromsize;
  unsigned long mmchdroffset;
  unsigned long mmchdrsize;
  unsigned long appdeviceinfooffset;
  unsigned long appdeviceinfosize;
  unsigned long frudataoffset;
  unsigned long frudatasize;
  unsigned long fpgacfgoffset;
  unsigned long fpgacfgsize;
  unsigned long sdrareaoffset;
  unsigned long sdrareasize;
  unsigned long pyldmgrareaoffset;
  unsigned long pyldmgrareasize;
  unsigned long adcscalingareaoffset;
  unsigned long adcscalingareasize;
} nvarea_info_t;

int main(int argc, char** argv) {
  bool fatal = false;
  int slotnum = 0;
  unsigned char retbytes[32];
  int retbytecnt;
  FILE* rfile;
  nvarea_info_t nvarea;
  

  // parse command line
  int i1 = 1;
  while ((i1<argc) && !fatal) {
    int argidx;
    string curargv = argv[i1];
    for (argidx=0; argidx<LASTARG; argidx++) {
      if (curargv == arglist[argidx]) {
        // have match
        uint tempui;
        switch (argidx) {
          case HELP:
            cout << "command line arguments:\n";
            cout << "  ? - display this text\n";
            cout << " -s <slot#> - MicroTCA slot number (1-13)\n";
            exit(0);
          case SLOT:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &slotnum);
            }
            break;
          default:
            cerr << "?fatal error while parsing command line--try running command again with '?' argument\n";
            fatal = true;
            break;
        }
        break;
      }
    }
    if (argidx>=LASTARG) {
      cerr << "?error, unknown argument '" << curargv << "'\n";    
      fatal = true;
      break;
    }
    i1++;
  }
  
  if ((slotnum < 1) || (slotnum >13)) {
    cerr << "?error invalid slot number (" << slotnum << ")\n";
    fatal = true;
  }

  if (fatal)
    // stop now, before things go too far
    exit(0);

  // read nonvolatile area info page 0
  sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x40 0x00 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
  system(cmdlinebuf);
  // open file and capture the returned bytes
  rfile = fopen(OUTPUTFILENAME, "r");
  if (!rfile) {
    cerr << "?error no results returned on IPMI command\n";
    exit(0);
  }
  retbytecnt = 0;
  while (fscanf(rfile, "%x", &retbytes[retbytecnt]) != EOF) {
    retbytecnt++;
  } 
  fclose(rfile);
  if (retbytecnt != 16) {
    cerr << "?error unexpected number of data bytes returned\n";
    exit(0);
  }
  nvarea.fmtversion = retbytes[0];
  nvarea.eepromsize = retbytes[3] << 8;
  nvarea.eepromsize += retbytes[2];
  nvarea.mmchdroffset = retbytes[5] << 8;
  nvarea.mmchdroffset += retbytes[4];
  nvarea.mmchdrsize = retbytes[6] * 8;
  nvarea.appdeviceinfooffset = retbytes[8] << 8;
  nvarea.appdeviceinfooffset += retbytes[7];
  nvarea.appdeviceinfosize = retbytes[9] * 8;
  nvarea.frudataoffset = retbytes[11] << 8;
  nvarea.frudataoffset += retbytes[10];
  nvarea.frudatasize = retbytes[12] * 8;
  nvarea.fpgacfgoffset = retbytes[14] << 8;
  nvarea.fpgacfgoffset += retbytes[13];
  nvarea.fpgacfgsize = retbytes[15] * 32;


  // read nonvolatile area info page 1
  sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x40 0x01 >%s\n", ipmbaddr[slotnum], OUTPUTFILENAME);
  system(cmdlinebuf);
  // open file and capture the returned bytes
  rfile = fopen(OUTPUTFILENAME, "r");
  if (!rfile) {
    cerr << "?error no results returned on IPMI command\n";
    exit(0);
  }
  retbytecnt = 0;
  while (fscanf(rfile, "%x", &retbytes[retbytecnt]) != EOF) {
    retbytecnt++;
  } 
  fclose(rfile);
  if (retbytecnt != 11) {
    cerr << "?error unexpected number of data bytes returned\n";
    exit(0);
  }
  nvarea.sdrareaoffset = retbytes[3] << 8;
  nvarea.sdrareaoffset += retbytes[2];
  nvarea.sdrareasize = retbytes[4] * 32;
  nvarea.pyldmgrareaoffset = retbytes[6] << 8;
  nvarea.pyldmgrareaoffset += retbytes[5];
  nvarea.pyldmgrareasize = retbytes[7] * 8;
  nvarea.adcscalingareaoffset = retbytes[9] << 8;
  nvarea.adcscalingareaoffset += retbytes[8];
  nvarea.adcscalingareasize = retbytes[10] * 8;

  cout << "Nonvolatile Area Summary:\n";
  sprintf(cmdlinebuf, "Area Version:  %i\n", nvarea.fmtversion);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "EEPROM Size:  0x%04x bytes\n", nvarea.eepromsize);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "Hardware ID Area Offset:  0x%04x   Size:  0x%04x bytes\n", nvarea.mmchdroffset, nvarea.mmchdrsize);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "App Device Area  Offset:  0x%04x   Size:  0x%04x bytes\n", nvarea.appdeviceinfooffset, nvarea.appdeviceinfosize);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "FRU Data Area    Offset:  0x%04x   Size:  0x%04x bytes\n", nvarea.frudataoffset, nvarea.frudatasize);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "FPGA Config Area Offset:  0x%04x   Size:  0x%04x bytes\n", nvarea.fpgacfgoffset, nvarea.fpgacfgsize);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "SDR Area         Offset:  0x%04x   Size:  0x%04x bytes\n", nvarea.sdrareaoffset, nvarea.sdrareasize);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "Payload Mgr Area Offset:  0x%04x   Size:  0x%04x bytes\n", nvarea.pyldmgrareaoffset, nvarea.pyldmgrareasize);  cout << cmdlinebuf;
  sprintf(cmdlinebuf, "ADC Scaling Area Offset:  0x%04x   Size:  0x%04x bytes\n", nvarea.adcscalingareaoffset, nvarea.adcscalingareasize);  cout << cmdlinebuf;




  return 1;
}
