
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>




using namespace std;

#define OUTPUTFILENAME		"%%tmp_ipmitool_output"

const int ipmbaddr[] = {0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0xa4};
const char* arglist[] = {"?", "-s", "-autopwr", "-btmode", "-fpgacfg", "-shtdnev", "-fltmsk"};
enum argtype {HELP=0, SLOT, AUTOPWR, BTMODE, FPGACFG, SHTDNEV, FLTMSK, LASTARG};
char cmdlinebuf[256];


int main(int argc, char** argv) {
  bool fatal = false;
  int slotnum = 0;
  unsigned char retbytes[32];
  int retbytecnt;
  FILE* rfile;
  unsigned char parammask = 0;
  unsigned char autopwrena = 0;
  unsigned char bootmode = 0;
  unsigned char fpgaautocfginhibit = 0;
  unsigned char shtdneventena = 0;
  unsigned char faultmasklevel = 0;

  // parse command line
  int i1 = 1;
  while ((i1<argc) && !fatal) {
    int argidx;
    string curargv = argv[i1];
    for (argidx=0; argidx<LASTARG; argidx++) {
      if (curargv == arglist[argidx]) {
        // have match
        uint tempui;
        switch (argidx) {
          case HELP:
            cout << "command line arguments:\n";
            cout << "  ? - display this text\n";
            cout << "  -s <slot#> - MicroTCA slot number (1-13)\n";
            cout << "  -autopwr on|off\n";
            cout << "  -btmode m|t\n"; 
            cout << "  -fpgacfg on|off\n"; 
            cout << "  -shtdnev on|off\n"; 
            cout << "  -fltmsk 0-3\n";
            exit(0);
          case SLOT:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              sscanf(argv[i1], "%i", &slotnum);
            }
            break;

          case AUTOPWR:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              if (string(argv[i1]) == "on") {
                parammask |= 0x80;
                autopwrena = 1;
              }
              else {
                if (string(argv[i1]) == "off") {
                  parammask |= 0x80;
                  autopwrena = 0;
                }
                else {
                  cerr << "?error unknown value '" << argv[i1] << "' on switch " << curargv << "\n";
                  fatal = true;
                }
              }
            }
            break;

          case BTMODE:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              if (string(argv[i1]) == "m") {
                parammask |= 0x40;
                bootmode = 1;
              }
              else {
                if (string(argv[i1]) == "t") {
                  parammask |= 0x40;
                  bootmode = 0;
                }
                else {
                  cerr << "?error unknown value '" << argv[i1] << "' on switch " << curargv << "\n";
                  fatal = true;
                }
              }
            }
            break;

          case FPGACFG:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              if (string(argv[i1]) == "off") {
                parammask |= 0x20;
                fpgaautocfginhibit = 1;
              }
              else {
                if (string(argv[i1]) == "on") {
                  parammask |= 0x20;
                  fpgaautocfginhibit = 0;
                }
                else {
                  cerr << "?error unknown value '" << argv[i1] << "' on switch " << curargv << "\n";
                  fatal = true;
                }
              }
            }
            break;

          case SHTDNEV:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              if (string(argv[i1]) == "on") {
                parammask |= 0x10;
                shtdneventena = 1;
              }
              else {
                if (string(argv[i1]) == "off") {
                  parammask |= 0x10;
                  shtdneventena = 0;
                }
                else {
                  cerr << "?error unknown value '" << argv[i1] << "' on switch " << curargv << "\n";
                  fatal = true;
                }
              }
            }
            break;

          case FLTMSK:
            i1++;
            if (i1 >= argc) {
              cerr << "?error missing slot number argument on command line\n";
              fatal = true;
            }
            else { 
              parammask |= 0x08;
              sscanf(argv[i1], "%i", &faultmasklevel);
              if (faultmasklevel > 3) {
                cerr << "?error unknown value '" << argv[i1] << "' on switch " << curargv << "\n";
                fatal = true;
              }
            }
            break;

          default:
            cerr << "?fatal error while parsing command line--try running command again with '?' argument\n";
            fatal = true;
            break;
        }
        break;
      }
    }
    if (argidx>=LASTARG) {
      cerr << "?error, unknown argument '" << curargv << "'\n";    
      fatal = true;
      break;
    }
    i1++;
  }
  
  if ((slotnum < 1) || (slotnum >13)) {
    cerr << "?error invalid slot number (" << slotnum << ")\n";
    fatal = true;
  }

  if (fatal)
    // stop now, before things go too far
    exit(0);

  // write the command 
  sprintf(cmdlinebuf, "ipmitool $IPMITOOLARGS -t 0x%02x raw 0x32 0x03 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x  >%s\n", ipmbaddr[slotnum], parammask,
    autopwrena, bootmode, fpgaautocfginhibit, shtdneventena, faultmasklevel, OUTPUTFILENAME);
  system(cmdlinebuf);

  return 1;
}
